/*

Simple program to rotate Apache logs without having to kill the server.

Original by Ben Laurie <ben@algroup.co.uk>
12 Mar 1996

Enhanced by Z. Wagner - Ice Bear Soft <wagner@mbox.cesnet.cz>
4 Mar 1998

28 Jul 1999: Added USR1 handling

Modified by F. Cipriani to close the log file every new day.
28 Mar 2000

*/

#define BUFSIZE		65536
#define MAX_PATH	1024

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <locale.h>
#include <signal.h>

void USR1handler(int sig, int reason) {
exit(0);
}

int main(int argc, char **argv) {
    char buf[BUFSIZE], buf2[MAX_PATH];
    time_t tLogEnd;
    time_t tRotation;
    int nLogFD=-1;
    int nRead;
    char *szLogRoot;

    // FC New variables 
    time_t curTime;
    struct tm *curTm;
    int curDay = -1;
    int lastDay = -1;

    if(argc < 3 || argc > 4) {
	fprintf(stderr,"%s <logfile template> <rotation time in seconds> [locale]\n\n",argv[0]);
  	fprintf(stderr,"Add this:\n\nTransferLog \"|%s /some/where.%%Y%%m%%d 86400\"\n\n",argv[0]);
	fprintf(stderr,"to httpd.conf. The generated name will be /some/where.nnnn where nnnn is the\n");
	fprintf(stderr,"system time at which the log nominally starts (N.B. this time will always be a\n");
        fprintf(stderr,"multiple of the rotation time, so you can synchronize cron scripts with it).\n");
        fprintf(stderr,"At the end of each rotation time a new log is started.\n");
        fprintf(stderr,"The time is converted according to the format specification for strftime\n");
        fprintf(stderr,"given in the logfile template using the optional LC_TIME locale.\n");
        fprintf(stderr,"It can also be used for ErrorLog");
        exit(1);
    }

    signal(SIGUSR1, USR1handler);

    szLogRoot=argv[1];
    tRotation=atoi(argv[2]);
    if(tRotation <= 0) {
        fprintf(stderr,"Rotation time must be > 0\n");
        exit(6);
    }
    tzset();
    if (argc == 4) setlocale(LC_TIME, argv[3]);

    for( ; ; ) {
        nRead=read(0,buf,sizeof buf);

        if(nRead == 0)
	    exit(3);

        if(nRead < 0)
            if (errno != EINTR)
                exit(4);

        // FC
        curTime = time(NULL);
        curTm = localtime(&curTime);
        curDay = curTm->tm_mday;

        // FC (last condition)
        if(nLogFD >= 0 && (curTime >= tLogEnd || nRead < 0 || curDay != lastDay)) {
            close(nLogFD);
            nLogFD=-1;
        }

        //FC
        lastDay = curDay;

        if (nLogFD < 0) {
            // FC, deleted two rows reusing calculated time

            strftime(buf2, sizeof buf2, szLogRoot, curTm);
	    tLogEnd= curTime + tRotation;
	    nLogFD=open(buf2,O_WRONLY|O_CREAT|O_APPEND,0666);
            if(nLogFD < 0) {
                perror(buf2);
                exit(2);
            }
        }
	if(write(nLogFD,buf,nRead) != nRead) {
            perror(buf2);
            exit(5);
        }
    }

    return 0;
}
